--[[
    BGLive is an user interface for spectated battleground 
	wargames in World of Warcraft.
    Copyright (C) 2015  Harald Böhm <harald@boehm.agency>
	Further contributors: Jochen Taeschner and Romina Schmidt.
	
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	ADDITIONAL PERMISSION UNDER GNU GPL VERSION 3 SECTION 7:
	As a special exception, the copyright holder of this add-on gives you
	permission to link this add-on with independent proprietary software,
	regardless of the license terms of the independent proprietary software.
]]

-- ArenaLive addon Name and localisation table:
local addonName, L = ...;

--[[
**************************************************
******* GENERAL HANDLER SET UP STARTS HERE *******
**************************************************
]]--
local SpecIcon = ArenaLive:ConstructHandler("TalentSpecialisationIcon", true);
SpecIcon:RegisterEvent("AL_SPEC_PLAYER_SPECIALIZATION_UPDATE");
SpecIcon:RegisterEvent("UPDATE_BATTLEFIELD_STATUS");

--[[
****************************************
****** HANDLER METHODS START HERE ******
****************************************
]]--
function SpecIcon:Update(unitFrame)
	local unit = unitFrame.unit;
	local icon = unitFrame[self.name];
	if ( not unit or not icon ) then
		return;
	end
	
	local specID = ArenaLiveSpectator.UnitCache:GetUnitSpecialisation(unit);
	if ( specID ) then
		local _, _, _, texture = GetSpecializationInfoByID(specID);
		if ( texture ) then
			icon:Show();
			icon:SetTexture(texture);
		else
			self:Reset(unitFrame);
		end
	else
		self:Reset(unitFrame);
	end
end


function SpecIcon:Reset(unitFrame)
	local icon = unitFrame[self.name];
	icon:Hide();
end

function SpecIcon:OnEvent(event, ...)
	local filter  = ...;
	if ( event == "AL_SPEC_PLAYER_SPECIALIZATION_UPDATE" and self.enabled ) then
		if ( ArenaLive:IsUnitInUnitFrameCache(filter) ) then
			for id in ArenaLive:GetAffectedUnitFramesByUnit(filter) do
				local unitFrame = ArenaLive:GetUnitFrameByID(id);
				if ( unitFrame[self.name] ) then
					SpecIcon:Update(unitFrame);
				end
			end
		end
	elseif ( event == "UPDATE_BATTLEFIELD_STATUS" ) then
		local status = GetBattlefieldStatus(filter);
		local isSpectator, queueType = IsSpectator();
		if( status == "active" and isSpectator and queueType == "battleground" ) then
			self.enabled = true;
			for id, unitFrame in ArenaLive:GetAllUnitFrames() do
				if ( frame.enabled and unitFrame[self.name] ) then
					SpecIcon:Update(unitFrame);
				end
			end
		elseif ( status == "none" ) then
			self.enabled = false;
		end
	end
end